from django.test import TestCase
from django.http.request import MultiValueDict

from ..backend import utils
from ..forms import WooeyForm
from ..forms import config as forms_config

from . import factories
from . import config
from . import mixins


class FormTestCase(mixins.ScriptFactoryMixin, mixins.FileCleanupMixin, TestCase):

    @staticmethod
    def get_mvdict(data):
        return MultiValueDict(data)

    def test_master_form(self):
        script_version = self.translate_script
        form = utils.get_master_form(script_version=script_version)
        self.assertTrue(isinstance(form, WooeyForm))
        qdict = self.get_mvdict(config.SCRIPT_DATA['translate'].get('data'))
        utils.validate_form(form=form, data=qdict,
                            files=config.SCRIPT_DATA['translate'].get('files'))
        self.assertTrue(form.is_valid())

    def test_group_form(self):
        script_version = self.translate_script
        form = utils.get_form_groups(script_version=script_version)
        self.assertEqual(len(form['groups']), 1)

    def test_multiplechoice_form(self):
        script_version = self.choice_script
        form = utils.get_master_form(script_version=script_version)
        # check our wrapper is in the form render
        form_str = str(form)
        self.assertTrue(forms_config.WOOEY_MULTI_WIDGET_ANCHOR in form_str)
        self.assertTrue(forms_config.WOOEY_MULTI_WIDGET_ATTR in form_str)
        qdict = self.get_mvdict(config.SCRIPT_DATA['choices'].get('data'))
        fdict = config.SCRIPT_DATA['choices'].get('files')
        utils.validate_form(form=form, data=qdict,
                            files=fdict)
        self.assertTrue(form.is_valid())
        # test we can create a job from this form
        # this is implemented to put data and files in the same dictionary, so update it
        form.cleaned_data.update(fdict)
        job = utils.create_wooey_job(script_version_pk=script_version.pk, data=form.cleaned_data)
        # check the files are here
        file_param = 'multiple_file_choices'
        files = [i.value for i in job.get_parameters() if i.parameter.slug == file_param]
        self.assertEqual(len(files), len(fdict.get(file_param)))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0005_size_bytes'),
    ]

    operations = [
        migrations.AlterField(
            model_name='script',
            name='script_group',
            field=models.ForeignKey(blank=True, to='wooey.ScriptGroup', null=True),
        ),
    ]



from __future__ import absolute_import
__author__ = 'chris'
from django.forms.models import model_to_dict
from ..backend import utils


class UpdateScriptsMixin(object):
    pass
    # this method is no longer needed, as scripts are not cached
    # def save(self, **kwargs):
    #     super(UpdateScriptsMixin, self).save(**kwargs)
    #     # remove script from factory
    #     utils.update_form_factory(script_version=self.script_version)


class WooeyPy2Mixin(object):
    def __unicode__(self):
        return unicode(self.__str__())


# from
# http://stackoverflow.com/questions/1355150/django-when-saving-how-can-you-check-if-a-field-has-changed
class ModelDiffMixin(object):
    """
    A model mixin that tracks model fields' values and provide some useful api
    to know what fields have been changed.
    """

    def __init__(self, *args, **kwargs):
        super(ModelDiffMixin, self).__init__(*args, **kwargs)
        self.__initial = self._dict

    @property
    def diff(self):
        d1 = self.__initial
        d2 = self._dict
        diffs = [(k, (v, d2[k])) for k, v in d1.items() if v != d2[k]]
        return dict(diffs)

    @property
    def has_changed(self):
        return bool(self.diff)

    @property
    def changed_fields(self):
        return self.diff.keys()

    def get_field_diff(self, field_name):
        """
        Returns a diff for field if it's changed and None otherwise.
        """
        return self.diff.get(field_name, None)

    def save(self, *args, **kwargs):
        """
        Saves model and set initial state.
        """
        super(ModelDiffMixin, self).save(*args, **kwargs)
        self.__initial = self._dict

    @property
    def _dict(self):
        return model_to_dict(self, fields=[field.name for field in
                             self._meta.fields])



from __future__ import absolute_import

import os

from django.conf import settings

from celery import app as celery_app

app = celery_app.app_or_default()
# app = Celery('{{ project_name }}')

# Using a string here means the worker will not have to
# pickle the object when using Windows.
app.config_from_object('django.conf:settings')


@app.task(bind=True)
def debug_task(self):
    print('Request: {0!r}'.format(self.request))



import argparse
import sys

parser = argparse.ArgumentParser(description="Something")
parser.add_argument('--one-choice', choices=[0, 1, 2, 3], nargs=1)
parser.add_argument('--two-choices', choices=[0, 1, 2, 3], nargs=2)
parser.add_argument('--at-least-one-choice', choices=[0, 1, 2, 3], nargs='+')
parser.add_argument('--all-choices', choices=[0, 1, 2, 3], nargs='*')
parser.add_argument('--need-at-least-one-numbers', type=int, nargs='+', required=True)
parser.add_argument('--choices-str', nargs='+', type=str)
parser.add_argument('--multiple-file-choices', type=argparse.FileType('r'), nargs='*')
parser.add_argument('--more-multiple-file-choices', type=argparse.FileType('r'), nargs='*')

if __name__ == '__main__':
    args = parser.parse_args()
    sys.stdout.write('{}'.format(args.multiple_file_choices))



from . import version
import os
if version.DJANGO_VERSION >= version.DJ17:
    default_app_config = 'wooey.apps.WooeyConfig'
else:
    if os.environ.get('TESTING') != 'True':
        from . import settings as wooey_settings
        # we need to call from within wooey_settings so the celery/etc vars are setup
        if not wooey_settings.settings.configured:
            wooey_settings.settings.configure()
        from .apps import WooeyConfig
        WooeyConfig().ready()



from __future__ import absolute_import
import subprocess
import tarfile
import os
import zipfile
import six
import traceback
import time

from django.utils.text import get_valid_filename
from django.core.files.storage import default_storage
from django.core.files import File
from django.conf import settings
from django.db.transaction import atomic

from celery import Task
from celery import states
from celery import app
from celery.contrib import rdb

from . import settings as wooey_settings

celery_app = app.app_or_default()


class WooeyTask(Task):
    pass

    # def after_return(self, status, retval, task_id, args, kwargs, einfo):
    #     job, created = WooeyJob.objects.get_or_create(wooey_celery_id=task_id)
    #     job.content_type.wooey_celery_state = status
    #     job.save()

@celery_app.task(base=WooeyTask)
def submit_script(**kwargs):
    job_id = kwargs.pop('wooey_job')
    resubmit = kwargs.pop('wooey_resubmit', False)
    from .backend import utils
    from .models import WooeyJob, WooeyFile
    job = WooeyJob.objects.get(pk=job_id)

    command = utils.get_job_commands(job=job)
    if resubmit:
        # clone ourselves, setting pk=None seems hackish but it works
        job.pk = None

    # This is where the script works from -- it is what is after the media_root since that may change between
    # setups/where our user uploads are stored.
    cwd = job.get_output_path()

    abscwd = os.path.abspath(os.path.join(settings.MEDIA_ROOT, cwd))
    job.command = ' '.join(command)
    job.save_path = cwd

    utils.mkdirs(abscwd)
    # make sure we have the script, otherwise download it. This can happen if we have an ephemeral file system or are
    # executing jobs on a worker node.
    script_path = job.script_version.script_path
    if not utils.get_storage(local=True).exists(script_path.path):
        utils.get_storage(local=True).save(script_path.path, script_path.file)

    job.status = WooeyJob.RUNNING
    job.save()

    proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=abscwd)

    stdout, stderr = proc.communicate()

    # tar/zip up the generated content for bulk downloads
    def get_valid_file(cwd, name, ext):
        out = os.path.join(cwd, name)
        index = 0
        while os.path.exists(six.u('{}.{}').format(out, ext)):
            index += 1
            out = os.path.join(cwd, six.u('{}_{}').format(name, index))
        return six.u('{}.{}').format(out, ext)

    # fetch the job again in case the database connection was lost during the job or something else changed.
    job = WooeyJob.objects.get(pk=job_id)
    # if there are files generated, make zip/tar files for download
    if len(os.listdir(abscwd)):
        tar_out = get_valid_file(abscwd, get_valid_filename(job.job_name), 'tar.gz')
        tar = tarfile.open(tar_out, "w:gz")
        tar_name = os.path.splitext(os.path.splitext(os.path.split(tar_out)[1])[0])[0]
        tar.add(abscwd, arcname=tar_name)
        tar.close()

        zip_out = get_valid_file(abscwd, get_valid_filename(job.job_name), 'zip')
        zip = zipfile.ZipFile(zip_out, "w")
        arcname = os.path.splitext(os.path.split(zip_out)[1])[0]
        zip.write(abscwd, arcname=arcname)
        for root, folders, filenames in os.walk(os.path.split(zip_out)[0]):
            for filename in filenames:
                path = os.path.join(root, filename)
                if path == tar_out:
                    continue
                if path == zip_out:
                    continue
                try:
                    zip.write(path, arcname=os.path.join(arcname, filename))
                except:
                    stderr = '{}\n{}'.format(stderr, traceback.format_exc())
        try:
            zip.close()
        except:
            stderr = '{}\n{}'.format(stderr, traceback.format_exc())

        # save all the files generated as well to our default storage for ephemeral storage setups
        if wooey_settings.WOOEY_EPHEMERAL_FILES:
            for root, folders, files in os.walk(abscwd):
                for filename in files:
                    filepath = os.path.join(root, filename)
                    s3path = os.path.join(root[root.find(cwd):], filename)
                    remote = utils.get_storage(local=False)
                    exists = remote.exists(s3path)
                    filesize = remote.size(s3path) if exists else 0
                    if not exists or (exists and filesize == 0):
                        if exists:
                            remote.delete(s3path)
                        remote.save(s3path, File(open(filepath, 'rb')))
    utils.create_job_fileinfo(job)

    job.stdout = stdout
    job.stderr = stderr
    job.status = WooeyJob.COMPLETED
    job.save()

    return (stdout, stderr)



#!/usr/bin/env python

__author__ = 'chris'
import argparse
import os
import sys

parser = argparse.ArgumentParser(description="Create a nested heatmap from a delimited file.")
parser.add_argument('--tsv', help='The delimited file to plot.', type=argparse.FileType('r'), required=True)
parser.add_argument('--delimiter', help='The delimiter for fields. Default: tab', type=str, default='\t')
parser.add_argument('--major-index', help='The first column to group by.', type=str, required=True)
parser.add_argument('--minor-index', help='The second column to group by.', type=str, required=True)
parser.add_argument('--minor-cutoff', help='The minimum number of minor entries grouped to be considered.', type=int, default=2)
parser.add_argument('--log-normalize', help='Whether to log normalize data.', action='store_true')
parser.add_argument('--translate', help='Whether to translate data so the minimum value is zero.', action='store_true')


def main():
    args = parser.parse_args()
    import numpy as np
    import pandas as pd
    import seaborn as sns
    major_index = args.major_index
    minor_index = args.minor_index
    df = pd.read_table(args.tsv, index_col=[major_index, minor_index], sep=args.delimiter)
    df = np.log2(df) if args.log_normalize else df
    # set our undected samples to our lowest detection
    df[df==-1*np.inf] = df[df!=-1*np.inf].min().min()
    # translate our data so we have no negatives (which would screw up our addition and makes no biological sense)
    if args.translate:
        df+=abs(df.min().min())
    major_counts = df.groupby(level=[major_index]).count()
    # we only want to plot samples with multiple values in the minor index
    cutoff = args.minor_cutoff
    multi = df[df.index.get_level_values(major_index).isin(major_counts[major_counts>=cutoff].dropna().index)]

    # Let's select the most variable minor axis elements
    most_variable = multi.groupby(level=major_index).var().mean(axis=1).order(ascending=False)
    # and group by 20s
    for i in xrange(11):
        dat = multi[multi.index.get_level_values(major_index).isin(most_variable.index[10*i:10*(i+1)])]
        # we want to cluster by our major index, and then under these plot the values of our minor index
        major_dat = dat.groupby(level=major_index).sum()
        seaborn_map = sns.clustermap(major_dat, row_cluster=True, col_cluster=True)
        # now we keep this clustering, but recreate our data to fit the above clustering, with our minor
        # index below the major index (you can think of transcript levels under gene levels if you are
        # a biologist)
        merged_dat = pd.DataFrame(columns=[seaborn_map.data2d.columns])
        for major_val in seaborn_map.data2d.index:
            minor_rows = multi[multi.index.get_level_values(major_index)==major_val][seaborn_map.data2d.columns]
            major_row = major_dat.loc[major_val, ][seaborn_map.data2d.columns]
            merged_dat.append(major_row)
            merged_dat = merged_dat.append(major_row).append(minor_rows)
        merged_map = sns.clustermap(merged_dat, row_cluster=False, col_cluster=False)

        # recreate our dendrogram, this is undocumented and probably a hack but it works
        seaborn_map.dendrogram_col.plot(merged_map.ax_col_dendrogram)

        # for rows, I imagine at some point it will fail to fall within the major axis but fortunately
        # for this dataset it is not true
        seaborn_map.dendrogram_row.plot(merged_map.ax_row_dendrogram)
        merged_map.savefig('{}_heatmap_{}.png'.format(os.path.split(args.tsv.name)[1], i))

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

__author__ = 'Chris Mitchell'
# Code modified from http://stackoverflow.com/questions/20718819/downloading-images-from-google-search-using-python-gives-error

import argparse
import sys
import os
import imghdr
from urllib import FancyURLopener
import urllib2
import json

description = """
This will find you cats, and optionally, kitties.
"""

parser = argparse.ArgumentParser(description = description)
parser.add_argument('--count', help='The number of cats to find (max: 10)', type=int, default=1)
parser.add_argument('--kittens', help='Search for kittens.', action='store_true')
parser.add_argument('--breed', help='The breed of cat to find', type=str, choices=('lol', 'tabby', 'bengal', 'scottish', 'grumpy'))


# Start FancyURLopener with defined version
class MyOpener(FancyURLopener):
    version = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; it; rv:1.8.1.11)Gecko/20071127     Firefox/2.0.0.11'

myopener = MyOpener()


def main():
    args = parser.parse_args()
    searchTerm = 'kittens' if args.kittens else 'cats'
    cat_count = args.count if args.count < 10 else 10
    if args.breed:
        searchTerm += '%20{0}'.format(args.breed)

    # Notice that the start changes for each iteration in order to request a new set of images for each loop
    found = 0
    i = 0
    downloaded = set([])
    while found <= cat_count:
        i += 1
        template = 'https://ajax.googleapis.com/ajax/services/search/images?v=1.0&q={}&start={}&userip=MyIP'
        url = template.format(searchTerm, i+1)
        request = urllib2.Request(url, None, {'Referer': 'testing'})
        response = urllib2.urlopen(request)

        # Get results using JSON
        results = json.load(response)
        data = results['responseData']
        dataInfo = data['results']

        # Iterate for each result and get unescaped url
        for count, myUrl in enumerate(dataInfo, 1):
            if found > cat_count:
                break
            uurl = myUrl['unescapedUrl']
            filename = uurl.split('/')[-1]
            if filename in downloaded:
                continue
            myopener.retrieve(uurl, filename)
            if imghdr.what(filename):
                found += 1
                downloaded.add(filename)
            else:
                os.remove(filename)


if __name__ == "__main__":
    sys.exit(main())



# copied from django-compressor since I like their style
import os
import django
DEBUG = True

TEST_DIR = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'tests')

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'unique-snowflake'
    }
}

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'wooey',
]

STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
]

STATIC_URL = '/static/'

STATIC_ROOT = os.path.join(TEST_DIR, 'static')

MEDIA_URL = '/files/'
MEDIA_ROOT = os.path.join(TEST_DIR, 'media')

if django.VERSION[:2] < (1, 6):
    TEST_RUNNER = 'discover_runner.DiscoverRunner'

SECRET_KEY = "iufoj=mibkpdz*%bob952x(%49rqgv8gg45k36kjcg76&-y5=!"

PASSWORD_HASHERS = (
    'django.contrib.auth.hashers.UnsaltedMD5PasswordHasher',
)

MIDDLEWARE_CLASSES = []

ROOT_URLCONF = 'wooey.urls'

WOOEY_EPHEMERAL_FILES = True
WOOEY_CELERY = False
WOOEY_FILE_DIR = 'wooey_test'

if os.environ.get('WOOEY_TEST_S3'):
    STATICFILES_STORAGE = DEFAULT_FILE_STORAGE = 'wooey.wooeystorage.CachedS3BotoStorage'
    from boto.s3.connection import VHostCallingFormat

    INSTALLED_APPS += (
        'storages',
    )

    AWS_CALLING_FORMAT = VHostCallingFormat

    AWS_ACCESS_KEY_ID = os.environ.get('AWS_ACCESS_KEY_ID', '')
    AWS_SECRET_ACCESS_KEY = os.environ.get('AWS_SECRET_ACCESS_KEY', '')
    AWS_STORAGE_BUCKET_NAME = os.environ.get('AWS_STORAGE_BUCKET_NAME', '')
    AWS_AUTO_CREATE_BUCKET = True
    AWS_QUERYSTRING_AUTH = False
    AWS_S3_FILE_OVERWRITE = False
    AWS_PRELOAD_METADATA = True
    AWS_S3_CUSTOM_DOMAIN = os.environ.get('AWS_S3_CUSTOM_DOMAIN', '')

    GZIP_CONTENT_TYPES = (
        'text/css',
        'application/javascript',
        'application/x-javascript',
        'text/javascript',
    )

    AWS_EXPIREY = 60 * 60 * 7
    AWS_HEADERS = {
        'Cache-Control': 'max-age=%d, s-maxage=%d, must-revalidate' % (AWS_EXPIREY,
            AWS_EXPIREY)
    }

    STATIC_URL = 'http://%s.s3.amazonaws.com/' % AWS_STORAGE_BUCKET_NAME
    MEDIA_URL = '/user-uploads/'

else:
    STATICFILES_STORAGE = DEFAULT_FILE_STORAGE = 'wooey.wooeystorage.FakeRemoteStorage'



from django.db import models
from django.conf import settings
from ..django_compat import GenericForeignKey
from django.contrib.contenttypes.models import ContentType


class Favorite(models.Model):

    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='favorites', blank=True, null=True)

    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    created_date = models.DateTimeField(auto_now_add=True)

    class Meta:
        unique_together = ('user', 'content_type', 'object_id', )



import os

from django.test import TestCase

from . import config
from ..backend import utils
from . import mixins


class FormTestCase(mixins.ScriptFactoryMixin, TestCase):

    def test_addscript(self):
        from django.core.management import call_command
        call_command('addscript', os.path.join(config.WOOEY_TEST_SCRIPTS, 'command_order.py'))



import factory
import os
import six
import uuid

from django.contrib.auth import get_user_model
from django.core.files.storage import default_storage

from ..models import WooeyJob, ScriptGroup, Script, ScriptParameter, ScriptParameterGroup, ScriptParameters

from .. import settings as wooey_settings


class ScriptGroupFactory(factory.DjangoModelFactory):
    class Meta:
        model = ScriptGroup

    group_name = 'test group'
    group_description = 'test desc'


class ScriptFactory(factory.DjangoModelFactory):
    class Meta:
        model = Script

    script_name = 'test script'
    script_group = factory.SubFactory(ScriptGroupFactory)
    script_description = 'test script desc'


class UserFactory(factory.DjangoModelFactory):
    class Meta:
        model = get_user_model()

    username = 'user'
    email = 'a@a.com'
    password = 'testuser'


class BaseJobFactory(factory.DjangoModelFactory):
    class Meta:
        model = WooeyJob
    job_name = six.u('\xd0\xb9\xd1\x86\xd1\x83')
    job_description = six.u('\xd0\xb9\xd1\x86\xd1\x83\xd0\xb5\xd0\xba\xd0\xb5')


def generate_script(script_path):
    filename = os.path.join(script_path)[1]
    filename = os.path.join(wooey_settings.WOOEY_SCRIPT_DIR, filename)
    new_file = default_storage.save(filename, open(script_path))
    from ..backend import utils
    res = utils.add_wooey_script(script_path=new_file, group=None)
    return res['script']


def generate_job(script_version):
    return BaseJobFactory(script_version=script_version)



from __future__ import absolute_import
from django import forms


class WooeyForm(forms.Form):

    def add_wooey_fields(self):
        # This adds fields such as job name, description that we like to validate on but don't want to include in
        # form rendering
        self.fields['job_name'] = forms.CharField()
        self.fields['job_description'] = forms.CharField(required=False)



# __author__ = 'chris'
# from django.views.generic import CreateView, UpdateView
# from django.forms import modelform_factory
# from django.core.urlresolvers import reverse_lazy
# from django.conf import settings
#
#
# class WooeyScriptMixin(object):
#     def dispatch(self, request, *args, **kwargs):
#         import pdb; pdb.set_trace();
#         self.script_name = kwargs.pop('script_name', None)
#         if not self.script_name:
#             return super(WooeyScriptMixin, self).dispatch(request, *args, **kwargs)
#         klass = getattr(self.wooey_models, self.script_name)
#         self.model = klass
#         return super(WooeyScriptMixin, self).dispatch(request, *args, **kwargs)
#
#     def get_queryset(self):
#         klass = getattr(self.wooey_models, self.script_name)
#         return klass.objects.all()
#
#     def get_context_data(self, **kwargs):
#         ctx = super(WooeyScriptMixin, self).get_context_data(**kwargs)
#         ctx['script_name'] = self.script_name
#         return ctx
#
#     def get_form_class(self):
#         return modelform_factory(self.model, fields=self.fields, exclude=('wooey_script_name', 'wooey_celery_id', 'wooey_celery_state'))
#
#
# class WooeyScriptEdit(WooeyScriptMixin, UpdateView):
#     template_name = 'generic_script_view.html'
#
#
# class WooeyScriptCreate(WooeyScriptMixin, CreateView):
#     template_name = 'generic_script_create.html'
#
#     def get_success_url(self):
#         return reverse_lazy(getattr(settings, 'POST_SCRIPT_URL', '{}_home'.format(self.app_name)))
#
#     def get_form_class(self):
#         return modelform_factory(self.model, fields=self.fields, exclude=('wooey_user', 'wooey_script_name', 'wooey_celery_id', 'wooey_celery_state'))
#



import shutil
import os

from ..models import ScriptVersion, WooeyFile, WooeyJob
from ..backend import utils
from .. import settings as wooey_settings
from . import factories, config


class FileCleanupMixin(object):
    def tearDown(self):
        for i in WooeyFile.objects.all():
            path = i.filepath.name
            utils.get_storage().delete(path)
            if wooey_settings.WOOEY_EPHEMERAL_FILES:
                utils.get_storage(local=False).delete(path)
        # delete job dirs
        local_storage = utils.get_storage(local=True)
        for i in WooeyJob.objects.all():
            shutil.rmtree(local_storage.path(i.get_output_path()))
        super(FileCleanupMixin, self).tearDown()


class ScriptFactoryMixin(object):
    def tearDown(self):
        for i in ScriptVersion.objects.all():
            path = i.script_path.name
            # import pdb; pdb.set_trace();
            utils.get_storage().delete(path)
            if wooey_settings.WOOEY_EPHEMERAL_FILES:
                utils.get_storage(local=False).delete(path)
            path += 'c'  # handle pyc junk
            utils.get_storage().delete(path)
        super(ScriptFactoryMixin, self).tearDown()

    def setUp(self):
        self.translate_script = factories.generate_script(os.path.join(config.WOOEY_TEST_SCRIPTS, 'translate.py'))
        self.choice_script = factories.generate_script(os.path.join(config.WOOEY_TEST_SCRIPTS, 'choices.py'))
        super(ScriptFactoryMixin, self).setUp()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import autoslug.fields
from django.conf import settings
import wooey.models.mixins


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='WooeyFile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('filepath', models.FileField(max_length=500, upload_to=b'')),
                ('filepreview', models.TextField(null=True, blank=True)),
                ('filetype', models.CharField(max_length=255, null=True, blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='WooeyJob',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('celery_id', models.CharField(max_length=255, null=True)),
                ('job_name', models.CharField(max_length=255)),
                ('job_description', models.TextField(null=True, blank=True)),
                ('stdout', models.TextField(null=True, blank=True)),
                ('stderr', models.TextField(null=True, blank=True)),
                ('status', models.CharField(default=b'submitted', max_length=255, choices=[(b'submitted', 'Submitted'), (b'running', 'Running'), (b'completed', 'Completed'), (b'deleted', 'Deleted')])),
                ('save_path', models.CharField(max_length=255, null=True, blank=True)),
                ('command', models.TextField()),
                ('created_date', models.DateTimeField(auto_now_add=True)),
                ('modified_date', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='Script',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('script_name', models.CharField(max_length=255)),
                ('slug', autoslug.fields.AutoSlugField(unique=True, editable=False)),
                ('script_description', models.TextField(null=True, blank=True)),
                ('script_order', models.PositiveSmallIntegerField(default=1)),
                ('is_active', models.BooleanField(default=True)),
                ('script_path', models.FileField(upload_to=b'')),
                ('execute_full_path', models.BooleanField(default=True)),
                ('save_path', models.CharField(help_text=b'By default save to the script name, this will change the output folder.', max_length=255, null=True, blank=True)),
                ('script_version', models.PositiveSmallIntegerField(default=0)),
                ('created_date', models.DateTimeField(auto_now_add=True)),
                ('modified_date', models.DateTimeField(auto_now=True)),
            ],
            bases=(wooey.models.mixins.ModelDiffMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ScriptGroup',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('group_name', models.TextField()),
                ('slug', autoslug.fields.AutoSlugField(unique=True, editable=False)),
                ('group_description', models.TextField(null=True, blank=True)),
                ('group_order', models.SmallIntegerField(default=1)),
                ('is_active', models.BooleanField(default=True)),
                ('user_groups', models.ManyToManyField(to='auth.Group', blank=True)),
            ],
            bases=(wooey.models.mixins.UpdateScriptsMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ScriptParameter',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('short_param', models.CharField(max_length=255)),
                ('script_param', models.CharField(max_length=255)),
                ('slug', autoslug.fields.AutoSlugField(unique=True, editable=False)),
                ('is_output', models.BooleanField(default=None)),
                ('required', models.BooleanField(default=False)),
                ('output_path', models.FilePathField(path=b'', max_length=255, allow_files=False, recursive=True, allow_folders=True)),
                ('choices', models.CharField(max_length=255, null=True, blank=True)),
                ('choice_limit', models.PositiveSmallIntegerField(null=True, blank=True)),
                ('form_field', models.CharField(max_length=255)),
                ('default', models.CharField(max_length=255, null=True, blank=True)),
                ('input_type', models.CharField(max_length=255)),
                ('param_help', models.TextField(null=True, verbose_name=b'help', blank=True)),
                ('is_checked', models.BooleanField(default=False)),
            ],
            bases=(wooey.models.mixins.UpdateScriptsMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ScriptParameterGroup',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('group_name', models.TextField()),
                ('script', models.ForeignKey(to='wooey.Script')),
            ],
            bases=(wooey.models.mixins.UpdateScriptsMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ScriptParameters',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('_value', models.TextField(db_column='value')),
                ('job', models.ForeignKey(to='wooey.WooeyJob')),
                ('parameter', models.ForeignKey(to='wooey.ScriptParameter')),
            ],
        ),
        migrations.AddField(
            model_name='scriptparameter',
            name='parameter_group',
            field=models.ForeignKey(to='wooey.ScriptParameterGroup'),
        ),
        migrations.AddField(
            model_name='scriptparameter',
            name='script',
            field=models.ForeignKey(to='wooey.Script'),
        ),
        migrations.AddField(
            model_name='script',
            name='script_group',
            field=models.ForeignKey(to='wooey.ScriptGroup'),
        ),
        migrations.AddField(
            model_name='script',
            name='user_groups',
            field=models.ManyToManyField(to='auth.Group', blank=True),
        ),
        migrations.AddField(
            model_name='wooeyjob',
            name='script',
            field=models.ForeignKey(to='wooey.Script'),
        ),
        migrations.AddField(
            model_name='wooeyjob',
            name='user',
            field=models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True),
        ),
        migrations.AddField(
            model_name='wooeyfile',
            name='job',
            field=models.ForeignKey(to='wooey.WooeyJob'),
        ),
        migrations.AddField(
            model_name='wooeyfile',
            name='parameter',
            field=models.ForeignKey(blank=True, to='wooey.ScriptParameters', null=True),
        ),
    ]



from __future__ import absolute_import
__author__ = 'chris'
import copy
import json
import six
import os
from collections import OrderedDict

from django import forms
from django.http.request import QueryDict
from django.utils.safestring import mark_safe

from .scripts import WooeyForm
from . import config
from ..backend import utils
from ..models import ScriptParameter, ScriptVersion
from ..django_compat import flatatt, format_html
from .. import version


def mutli_render(render_func, appender_data_dict=None):
    def render(name, values, attrs=None):
        if not isinstance(values, (list, tuple)):
            values = [values]
        # The tag is a marker for our javascript to reshuffle the elements. This is because some widgets have complex rendering with multiple fields
        pieces = ['<{tag} {multi_attr}>{widget}</{tag}>'.format(tag='div', multi_attr=config.WOOEY_MULTI_WIDGET_ATTR,
                                                                widget=render_func(name, value, attrs)) for value in values]

        # we add a final piece that is our button to click for adding. It's useful to have it here instead of the template so we don't
        # have to reverse-engineer who goes with what

        # build the attribute dict
        data_attrs = flatatt(appender_data_dict if appender_data_dict is not None else {})
        pieces.append(format_html('<a href="#{anchor}"{data}><span class="glyphicon glyphicon-plus"></span></a>', anchor=config.WOOEY_MULTI_WIDGET_ANCHOR
                                  , data=data_attrs))
        return mark_safe('\n'.join(pieces))
    return render


def multi_value_from_datadict(func):
    def value_from_datadict(data, files, name):
        return [func(QueryDict('{name}={value}'.format(name=name, value=i)), files, name) for i in data.getlist(name)]
    return value_from_datadict


def multi_value_clean(func):
    def clean(*args, **kwargs):
        args = list(args)
        values = args[0]
        ret = []
        for value in values:
            value_args = args
            value_args[0] = value
            ret.append(func(*value_args, **kwargs))
        return ret
    return clean


class WooeyFormFactory(object):
    wooey_forms = {}

    @staticmethod
    def get_field(param, initial=None):
        """
        Any extra field attributes for the widget for customization of Wooey at the field level
         can be added to the widget dictionary, widget_data_dict, or to the appender_data_dict, which
         is the little plus button. This is useful since there is only a single copy of the plus,
         whereas we can have multiple widgets. Thus, javascript attributes we want to add per parameter
         can be added to appender_data_dict, and attributes we want to add to the widget input itself
         can be added to the widget_data_dict.

        :return: a field class
        """
        form_field = param.form_field
        widget_data_dict = {}
        appender_data_dict = {}
        WOOEY_CHOICE_LIMIT = 'data-wooey-choice-limit'
        choices = json.loads(param.choices)
        field_kwargs = {'label': param.script_param.replace('_', ' ').title(),
                        'required': param.required,
                        'help_text': param.param_help,
                        'initial': param.default,
                        }
        multiple_choices = param.multiple_choice
        choice_limit = param.max_choices
        if choices:
            form_field = 'MultipleChoiceField' if multiple_choices else 'ChoiceField'
            base_choices = [(None, '----')] if not param.required and not multiple_choices else []
            field_kwargs['choices'] = base_choices+[(str(i), str(i).title()) for i in choices]
        if form_field == 'FileField':
            if param.is_output:
                form_field = 'CharField'
                if initial:
                    if not isinstance(initial, (list, tuple)):
                        initial = [initial]
                    initial = [os.path.split(i.name)[1] for i in initial]
            elif initial is not None and list(filter(None, initial)):  # for python3, we need to evaluate the filter object
                if isinstance(initial, (list, tuple)):
                    initial = [utils.get_storage_object(value) if not hasattr(value, 'path') else value for value in initial if value is not None]
                else:
                    initial = utils.get_storage_object(initial) if not hasattr(initial, 'path') else initial
                field_kwargs['widget'] = forms.ClearableFileInput()
        if not multiple_choices and isinstance(initial, list):
            initial = initial[0]
        field_kwargs['initial'] = initial
        field = getattr(forms, form_field)
        field = field(**field_kwargs)

        if form_field != 'MultipleChoiceField' and multiple_choices:
            field.widget.render = mutli_render(field.widget.render, appender_data_dict=appender_data_dict)
            field.widget.value_from_datadict = multi_value_from_datadict(field.widget.value_from_datadict)
            field.clean = multi_value_clean(field.clean)
            if choice_limit > 0:
                appender_data_dict[WOOEY_CHOICE_LIMIT] = choice_limit
        elif multiple_choices and choice_limit>0:
            widget_data_dict[WOOEY_CHOICE_LIMIT] = choice_limit
        field.widget.attrs.update(widget_data_dict)
        return field

    def get_group_forms(self, script_version=None, pk=None, initial_dict=None, render_fn=None):
        pk = int(pk) if pk is not None else pk
        if initial_dict is None:
            initial_dict = {}
        if pk is not None and pk in self.wooey_forms and initial_dict is None:
            if 'groups' in self.wooey_forms[pk]:
                if (version.PY_MINOR_VERSION == version.PY34 and version.PY_FULL_VERSION >= version.PY343) or \
                        (version.PY_MINOR_VERSION == version.PY33 and version.PY_FULL_VERSION >= version.PY336):
                    return copy.deepcopy(self.wooey_forms[pk]['groups'])
        params = ScriptParameter.objects.filter(script_version=script_version).order_by('pk')
        # set a reference to the object type for POST methods to use
        script_id_field = forms.CharField(widget=forms.HiddenInput)
        group_map = {}
        for param in params:
            initial_values = initial_dict.get(param.slug, None)
            field = self.get_field(param, initial=initial_values)
            field.name = param.slug
            group_id = -1 if param.required else param.parameter_group.pk
            group_name = 'Required' if param.required else param.parameter_group.group_name
            group = group_map.get(group_id, {
                'group': group_name,
                'fields': OrderedDict()
            })
            group['fields'][param.slug] = field
            group_map[group_id] = group
        # create individual forms for each group
        group_map = OrderedDict([(i, group_map[i]) for i in sorted(group_map.keys())])
        d = {'action': script_version.get_url()}
        d['groups'] = []
        pk = script_version.pk
        for group_index, group in enumerate(six.iteritems(group_map)):
            group_pk, group_info = group
            form = WooeyForm()
            if group_index == 0:
                form.fields['wooey_type'] = script_id_field
                form.fields['wooey_type'].initial = pk
            for field_pk, field in six.iteritems(group_info['fields']):
                form.fields[field_pk] = field

            if render_fn:
                form = render_fn(form)

            d['groups'].append({'group_name': group_info['group'], 'form': form})
        try:
            self.wooey_forms[pk]['groups'] = d
        except KeyError:
            self.wooey_forms[pk] = {'groups': d}
        # if the master form doesn't exist, create it while we have the model
        if 'master' not in self.wooey_forms[pk]:
            self.get_master_form(script_version=script_version, pk=pk)
        return d

    def get_master_form(self, script_version=None, pk=None):
        pk = int(pk) if pk is not None else pk
        if pk is not None and pk in self.wooey_forms:
            if 'master' in self.wooey_forms[pk]:
                if (version.PY_MINOR_VERSION == version.PY34 and version.PY_FULL_VERSION >= version.PY343) or \
                        (version.PY_MINOR_VERSION == version.PY33 and version.PY_FULL_VERSION >= version.PY336):
                    return copy.deepcopy(self.wooey_forms[pk]['master'])
        if script_version is None and pk is not None:
            script_version = ScriptVersion.objects.get(pk=pk)
        master_form = WooeyForm()
        params = ScriptParameter.objects.filter(script_version=script_version).order_by('pk')
        # set a reference to the object type for POST methods to use
        pk = script_version.pk
        script_id_field = forms.CharField(widget=forms.HiddenInput)
        master_form.fields['wooey_type'] = script_id_field
        master_form.fields['wooey_type'].initial = pk

        for param in params:
            field = self.get_field(param)
            master_form.fields[param.slug] = field
        try:
            self.wooey_forms[pk]['master'] = master_form
        except KeyError:
            self.wooey_forms[pk] = {'master': master_form}
        # create the group forms while we have the model
        if 'groups' not in self.wooey_forms[pk]:
            self.get_group_forms(script_version=script_version, pk=pk)
        return master_form

    def reset_forms(self, script_version=None):
        if script_version is not None and script_version.pk in self.wooey_forms:
            del self.wooey_forms[script_version.pk]

DJ_FORM_FACTORY = WooeyFormFactory()



import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='wooey',
    version='0.9.0',
    packages=find_packages(),
    scripts=['scripts/wooify'],
    entry_points={'console_scripts': ['wooify = wooey.backend.command_line:bootstrap', ]},
    install_requires = ['Django>=1.6', 'django-autoslug', 'django-celery', 'six', 'clinto>=0.1.1'],
    include_package_data=True,
    description='A Django app which creates a web GUI and task interface for argparse scripts',
    url='http://www.github.com/wooey/django-djangui',
    author='Chris Mitchell <chris.mit7@gmail.com>, Martin Fitzpatrick <martin.fitzpatrick@gmail.com>',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('wooey', '0004_choice_limits'),
    ]

    operations = [
        migrations.CreateModel(
            name='Favorite',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('object_id', models.PositiveIntegerField()),
                ('created_date', models.DateTimeField(auto_now_add=True)),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
                ('user', models.ForeignKey(related_name='favorites', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='favorite',
            unique_together=set([('user', 'content_type', 'object_id')]),
        ),
        migrations.AddField(
            model_name='wooeyfile',
            name='size_bytes',
            field=models.IntegerField(null=True),
        ),
    ]



__author__ = 'chris'
import traceback
import sys


class ProcessExceptionMiddleware(object):
    def process_response(self, request, response):
        if response.status_code != 200:
            try:
                sys.stderr.write('{}'.format(''.join(traceback.format_exc())))
            except AttributeError:
                pass
        return response



import os

from django.test import TestCase

from . import config
from ..templatetags import wooey_tags
from .. import settings as wooey_settings


class TemplateTagsTestCase(TestCase):

    def test_get_wooey_setting(self):
        #test that get_wooey_setting works as expected
        self.assertEqual(wooey_tags.get_wooey_setting("WOOEY_SITE_NAME"), wooey_settings.WOOEY_SITE_NAME)
        #test that get_wooey_setting works following a change
        wooey_settings.WOOEY_SITE_NAME = "TEST_SITE"
        self.assertEqual(wooey_tags.get_wooey_setting("WOOEY_SITE_NAME"), wooey_settings.WOOEY_SITE_NAME)



import os

from django.test import TestCase

from ..backend import utils

from . import factories
from . import config
from . import mixins


class TestUtils(mixins.ScriptFactoryMixin, TestCase):
    def test_sanitize_name(self):
        assert(utils.sanitize_name('abc')) == 'abc'
        assert(utils.sanitize_name('ab c')) == 'ab_c'
        assert(utils.sanitize_name('ab-c')) == 'ab_c'

    def test_sanitize_string(self):
        assert(utils.sanitize_string('ab"c')) == 'ab\\"c'

    def test_add_script(self):
        pass
        # TODO: fix me
        # utils.add_wooey_script(script=os.path.join(config.WOOEY_TEST_SCRIPTS, 'translate.py'))

    def test_anonymous_users(self):
        from .. import settings as wooey_settings
        from django.contrib.auth.models import AnonymousUser
        user = AnonymousUser()
        script_version = self.translate_script
        script = script_version.script
        d = utils.valid_user(script, user)
        self.assertTrue(d['valid'])
        wooey_settings.WOOEY_ALLOW_ANONYMOUS = False
        d = utils.valid_user(script, user)
        self.assertFalse(d['valid'])

    def test_valid_user(self):
        user = factories.UserFactory()
        script_version = self.translate_script
        script = script_version.script
        d = utils.valid_user(script, user)
        self.assertTrue(d['valid'])
        from .. import settings as wooey_settings
        self.assertEqual('disabled', d['display'])
        wooey_settings.WOOEY_SHOW_LOCKED_SCRIPTS = False
        d = utils.valid_user(script, user)
        self.assertEqual('hide', d['display'])
        from django.contrib.auth.models import Group
        test_group = Group(name='test')
        test_group.save()
        script.user_groups.add(test_group)
        d = utils.valid_user(script, user)
        self.assertFalse(d['valid'])
        user.groups.add(test_group)
        d = utils.valid_user(script, user)
        self.assertTrue(d['valid'])


class TestFileDetectors(TestCase):
    def test_detector(self):
        self.file = os.path.join(config.WOOEY_TEST_DATA, 'fasta.fasta')
        res, preview = utils.test_fastx(self.file)
        self.assertEqual(res, True, 'Fastx parser fail')
        self.assertEqual(preview, open(self.file).readlines(), 'Fastx Preview Fail')

    def test_delimited(self):
        self.file = os.path.join(config.WOOEY_TEST_DATA, 'delimited.tsv')
        res, preview = utils.test_delimited(self.file)
        self.assertEqual(res, True, 'Delimited parser fail')
        self.assertEqual(preview, [i.strip().split('\t') for i in open(self.file).readlines()], 'Delimited Preview Fail')



import os

from django.test import TestCase
from django.conf import settings

from . import config, mixins
from ..backend import utils
from ..models import ScriptVersion
from .. import settings as wooey_settings


class ScriptAdditionTests(mixins.ScriptFactoryMixin, TestCase):
    def setUp(self):
        self.storage = utils.get_storage(local=not wooey_settings.WOOEY_EPHEMERAL_FILES)
        self.filename_func = lambda x: os.path.join(wooey_settings.WOOEY_SCRIPT_DIR, x)

    def test_command_order(self):
        script = os.path.join(config.WOOEY_TEST_SCRIPTS, 'command_order.py')
        new_file = self.storage.save(self.filename_func('command_order.py'), open(script))
        res = utils.add_wooey_script(script_path=new_file, group=None)
        self.assertEqual(res['valid'], True, res['errors'])
        job = utils.create_wooey_job(script_version_pk=1, data={'job_name': 'abc', 'link': 'alink', 'name': 'aname'})
        # These are positional arguments -- we DO NOT want them returning anything
        self.assertEqual(['', ''], [i.parameter.short_param for i in job.get_parameters()])
        # These are the params shown to the user, we want them returning their destination
        # This also checks that we maintain the expected order
        self.assertEqual(['link', 'name'], [i.parameter.script_param for i in job.get_parameters()])
        # Check the job command
        commands = utils.get_job_commands(job=job)[2:]
        self.assertEqual(['alink', 'aname'], commands)

    def test_script_upgrade(self):
        script_path = os.path.join(config.WOOEY_TEST_SCRIPTS, 'command_order.py')
        new_file = self.storage.save(self.filename_func('command_order.py'), open(script_path))
        res = utils.add_wooey_script(script_path=new_file, group=None)
        self.assertEqual(res['valid'], True, res['errors'])
        # upgrade script
        script = ScriptVersion.objects.get(pk=1)
        new_script = self.storage.save(self.filename_func('command_order.py'), open(script_path))
        script.script_path = new_script
        # we are going to be cloning this, so we lose the old object
        old_pk, old_iter = script.pk, script.script_iteration
        script.save()
        self.assertNotEqual(old_pk, script.pk)
        self.assertNotEqual(old_iter, script.script_iteration)
        # asset we are using the latest script in the frontend
        self.assertIn(script, [i.latest_version for i in utils.get_current_scripts()])
        old_script = ScriptVersion.objects.get(pk=old_pk)
        self.assertNotIn(old_script, [i.latest_version for i in utils.get_current_scripts()])
